@@ -172,31 +172,27 @@ xpad_pad_new_from_file (XpadPadGroup *group, XpadSettings *settings, const gchar
 	GtkWidget *pad = NULL;
 	gchar *content;
 
-	content = fio_get_file (filename);
+	content = fio_get_file (filename, CURRENT_WORK_DIR);
 
-	if (!content)
-	{
+	if (!content) {
 		gchar *usertext = g_strdup_printf (_("Could not read file %s."), filename);
 		xpad_app_error (NULL, usertext, NULL);
 		g_free (usertext);
-	}
-	else
-	{
+	} else {
 		GtkSourceBuffer *buffer;
 
-		xpad_periodic_init ();
-		xpad_periodic_set_callback ("save-content", (XpadPeriodicFunc) xpad_pad_save_content);
-
 		pad = xpad_pad_new (group, settings);
+
 		buffer = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (XPAD_PAD (pad)->priv->textview)));
 
 		xpad_text_buffer_freeze_undo (XPAD_TEXT_BUFFER (buffer));
-		g_signal_handlers_block_by_func (buffer, xpad_pad_text_changed, pad);
 
+		g_signal_handlers_block_by_func (buffer, xpad_pad_text_changed, pad);
 		xpad_text_buffer_set_text_with_tags (XPAD_TEXT_BUFFER (buffer), content ? content : "");
+		g_signal_handlers_unblock_by_func (buffer, xpad_pad_text_changed, pad);
+
 		g_free (content);
 
-		g_signal_handlers_unblock_by_func (buffer, xpad_pad_text_changed, pad);
 		xpad_text_buffer_thaw_undo (XPAD_TEXT_BUFFER (buffer));
 
 		xpad_pad_text_changed(XPAD_PAD(pad), buffer);
@@ -424,10 +420,9 @@ xpad_pad_dispose (GObject *object)
 {
 	XpadPad *pad = XPAD_PAD (object);
 
-	if (pad->priv->group) {
-		g_object_unref(pad->priv->group);
-		pad->priv->group = NULL;
-	}
+	xpad_pad_remove_accelerator_group (pad);
+
+	g_clear_object(&pad->priv->group);
 
 	if (GTK_IS_WIDGET(pad->priv->menu)) {
 		gtk_widget_destroy (pad->priv->menu);
@@ -463,8 +458,7 @@ xpad_pad_finalize (GObject *object)
 
 	if (pad->priv->settings) {
 		g_signal_handlers_disconnect_matched (pad->priv->settings, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, pad);
-		g_object_unref(pad->priv->settings);
-		pad->priv->settings = NULL;
+		g_clear_object(&pad->priv->settings);
 	}
 
 	g_free (pad->priv->infoname);
@@ -635,7 +629,7 @@ xpad_pad_text_and_toolbar_height (XpadPad *pad)
 		return (guint) y + pad->priv->toolbar_height + gtk_container_get_border_width (GTK_CONTAINER (pad_textview));
 	}
 	else {
-		g_warning("There is a problem in the program Xpad. In function 'xpad_pad_toolbar_size_allocate' the variable 'event->height' is not a postive number. Please send a bugreport to https://bugs.launchpad.net/xpad/+filebug to help improve Xpad.");
+		g_warning("There is a problem in the program Xpad. In function 'xpad_pad_toolbar_size_allocate' the variable 'event->height' is not a positive number. Please send a bugreport to https://bugs.launchpad.net/xpad/+filebug to help improve Xpad.");
 		return 0;
 	}
 }
@@ -659,7 +653,7 @@ xpad_pad_show_toolbar (XpadPad *pad)
 				pad->priv->toolbar_height = (guint) req.height;
 			}
 			else {
-				g_warning ("There is a problem in the program Xpad. In function 'xpad_pad_show_toolbar' the variable 'req.height' is not a postive number. Please send a bugreport to https://bugs.launchpad.net/xpad/+filebug to help improve Xpad.");
+				g_warning ("There is a problem in the program Xpad. In function 'xpad_pad_show_toolbar' the variable 'req.height' is not a positive number. Please send a bugreport to https://bugs.launchpad.net/xpad/+filebug to help improve Xpad.");
 				pad->priv->toolbar_height = 0;
 			}
 		}
@@ -939,10 +933,10 @@ prop_notify_font (XpadPad *pad)
 	{
 		const gchar *font;
 		g_object_get (prop, "fontname", &font, NULL);
-		PangoFontDescription *fontdesc;
 
+		PangoFontDescription *fontdesc;
 		fontdesc = font ? pango_font_description_from_string (font) : NULL;
-		gtk_widget_override_font (pad->priv->textview, fontdesc);
+		xpad_text_view_set_font (pad->priv->textview, fontdesc);
 		if (fontdesc)
 			pango_font_description_free (fontdesc);
 	}
@@ -961,20 +955,15 @@ prop_notify_colors (XpadPad *pad)
 	g_object_get (prop, "follow-color-style", &follow_color_style, NULL);
 	g_object_set (XPAD_TEXT_VIEW (pad->priv->textview), "follow-color-style", follow_color_style, NULL);
 
-	if (follow_color_style)
+	if (follow_color_style) {
 		/* Set the colors to the global preferences colors */
 		g_object_get (pad->priv->settings, "text-color", &text_color, "back-color", &back_color, NULL);
-	else
+	} else {
 		/* Set the color to the individual pad properties colors */
 		g_object_get (prop, "text-color", &text_color, "back-color", &back_color, NULL);
+	}
 
-	gtk_widget_override_cursor (pad->priv->textview, text_color, text_color);
-	gtk_widget_override_color (pad->priv->textview, GTK_STATE_FLAG_NORMAL, text_color);
-	gtk_widget_override_background_color (pad->priv->textview, GTK_STATE_FLAG_NORMAL, back_color);
-
-	/* Inverse the text and background colors for selected text, so it is likely to be visible by any choice of the colors. */
-	gtk_widget_override_color (pad->priv->textview, GTK_STATE_FLAG_SELECTED, back_color);
-	gtk_widget_override_background_color (pad->priv->textview, GTK_STATE_FLAG_SELECTED, text_color);
+	xpad_text_view_set_colors (pad->priv->textview, text_color, back_color);
 
 	xpad_pad_save_info_delayed (pad);
 }
@@ -1005,7 +994,7 @@ xpad_pad_open_properties (XpadPad *pad)
 	style = gtk_widget_get_style_context (pad->priv->textview);
 	gtk_style_context_get(style, GTK_STATE_FLAG_NORMAL, GTK_STYLE_PROPERTY_FONT, &font, NULL);
 	gtk_style_context_get_color (style, GTK_STATE_FLAG_NORMAL, &widget_text_color);
-	gtk_style_context_get_background_color (style, GTK_STATE_FLAG_NORMAL, &widget_background_color);
+	get_background_color (style, GTK_STATE_FLAG_NORMAL, &widget_background_color);
 
 	g_object_get (XPAD_TEXT_VIEW (pad->priv->textview), "follow-font-style", &follow_font_style, "follow-color-style", &follow_color_style, NULL);
 	g_object_set (G_OBJECT (pad->priv->properties),
@@ -1055,7 +1044,7 @@ xpad_pad_toolbar_size_allocate (XpadPad *pad, GtkAllocation *event)
 		pad->priv->toolbar_height = (guint) event->height;
 	}
 	else {
-		g_warning("There is a problem in the program Xpad. In function 'xpad_pad_toolbar_size_allocate' the variable 'event->height' is not a postive number. Please send a bugreport to https://bugs.launchpad.net/xpad/+filebug to help improve Xpad.");
+		g_warning("There is a problem in the program Xpad. In function 'xpad_pad_toolbar_size_allocate' the variable 'event->height' is not a positive number. Please send a bugreport to https://bugs.launchpad.net/xpad/+filebug to help improve Xpad.");
 		pad->priv->toolbar_height = 0;
 	}
 	return FALSE;
@@ -1080,7 +1069,7 @@ xpad_pad_configure_event (XpadPad *pad, GdkEventConfigure *event)
 		}
 	}
 	else {
-		g_warning("There is a problem in the program Xpad. In function 'xpad_pad_configure_event' the variable 'event->width' or 'event->height' is not a postive number. Please send a bugreport to https://bugs.launchpad.net/xpad/+filebug to help improve Xpad.");
+		g_warning("There is a problem in the program Xpad. In function 'xpad_pad_configure_event' the variable 'event->width' or 'event->height' is not a positive number. Please send a bugreport to https://bugs.launchpad.net/xpad/+filebug to help improve Xpad.");
 	}
 
 	if (pad->priv->x != event->x || pad->priv->y != event->y) {
@@ -1183,24 +1172,22 @@ void
 xpad_pad_load_content (XpadPad *pad)
 {
 	g_return_if_fail (pad);
+	g_return_if_fail (pad->priv->contentname);
 
 	gchar *content;
 	GtkSourceBuffer *buffer;
 
-	if (!pad->priv->contentname)
-		return;
-
-	content = fio_get_file (pad->priv->contentname);
+	content = fio_get_file (pad->priv->contentname, CONFIG_DIR);
 
 	buffer = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (pad->priv->textview)));
 
 	xpad_text_buffer_freeze_undo (XPAD_TEXT_BUFFER (buffer));
+
 	g_signal_handlers_block_by_func (buffer, xpad_pad_text_changed, pad);
-
 	xpad_text_buffer_set_text_with_tags (XPAD_TEXT_BUFFER (buffer), content ? content : "");
-	g_free (content);
-
 	g_signal_handlers_unblock_by_func (buffer, xpad_pad_text_changed, pad);
+
+	g_free (content);
 	xpad_text_buffer_thaw_undo (XPAD_TEXT_BUFFER (buffer));
 
 	xpad_pad_text_changed(pad, buffer);
@@ -1215,8 +1202,9 @@ xpad_pad_save_content (XpadPad *pad)
 	gchar *content = NULL;
 	XpadTextBuffer *buffer;
 
-	if (!pad->priv->unsaved_content)
+	if (!pad->priv->unsaved_content) {
 		return;
+	}
 
 	/* create content file if it doesn't exist yet */
 	if (!pad->priv->contentname)
@@ -1229,9 +1217,9 @@ xpad_pad_save_content (XpadPad *pad)
 	if (GTK_IS_TEXT_VIEW(GTK_TEXT_VIEW (pad->priv->textview))) {
 		buffer = XPAD_TEXT_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (pad->priv->textview)));
 		content = xpad_text_buffer_get_text_with_tags (buffer);
-	}
-	else
+	} else {
 		g_warning("There is a problem in the program Xpad. In function 'xpad_pad_save_content' the variable 'pad->priv->textview' is not of type textview. Please send a bugreport to https://bugs.launchpad.net/xpad/+filebug to help improve Xpad.");
+	}
 
 	fio_set_file (pad->priv->contentname, content);
 
@@ -1251,7 +1239,7 @@ xpad_pad_load_info (XpadPad *pad, gboolean *show)
 	if (!pad->priv->infoname)
 		return;
 
-	if (fio_get_values_from_file (pad->priv->infoname, 
+	if (fio_get_values_from_file (pad->priv->infoname,
 		"i|width", &pad->priv->width,
 		"i|height", &pad->priv->height,
 		"i|x", &pad->priv->x,
@@ -1285,13 +1273,14 @@ xpad_pad_load_info (XpadPad *pad, gboolean *show)
 
 	g_object_set (XPAD_TEXT_VIEW (pad->priv->textview), "follow-font-style", follow_font, "follow-color-style", follow_color, NULL);
 
-	if (locked)
+	if (locked) {
 		g_object_set (XPAD_TEXT_VIEW (pad->priv->textview), "follow-font-style", FALSE, "follow-color-style", FALSE, NULL);
+	}
 
 	if (!follow_font)
 	{
 		PangoFontDescription *font_desc = pango_font_description_from_string (fontname);
-		gtk_widget_override_font (pad->priv->textview, font_desc);
+		xpad_text_view_set_font(pad->priv->textview, font_desc);
 		pango_font_description_free (font_desc);
 	}
 
@@ -1315,13 +1304,7 @@ xpad_pad_load_info (XpadPad *pad, gboolean *show)
 		}
 
 		/* Set the text and background color for this pad, as stated in its properties file. */
-		gtk_widget_override_cursor (pad->priv->textview, &text_color, &text_color);
-		gtk_widget_override_color (pad->priv->textview, GTK_STATE_FLAG_NORMAL, &text_color);
-		gtk_widget_override_background_color (pad->priv->textview, GTK_STATE_FLAG_NORMAL, &back_color);
-
-		/* Inverse the text and background colors for selected text, so it is likely to be visible by any choice of the colors. */
-		gtk_widget_override_color (pad->priv->textview, GTK_STATE_FLAG_SELECTED, &back_color);
-		gtk_widget_override_background_color (pad->priv->textview, GTK_STATE_FLAG_SELECTED, &text_color);
+		xpad_text_view_set_colors (pad->priv->textview, &text_color, &back_color);
 	}
 
 	if (pad->priv->sticky)
@@ -1372,7 +1355,7 @@ xpad_pad_save_info (XpadPad *pad)
 	style = gtk_widget_get_style_context (pad->priv->textview);
 	gtk_style_context_get (style, GTK_STATE_FLAG_NORMAL, GTK_STYLE_PROPERTY_FONT, &font, NULL);
 	gtk_style_context_get_color (style, GTK_STATE_FLAG_NORMAL, &text_color);
-	gtk_style_context_get_background_color (style, GTK_STATE_FLAG_NORMAL, &back_color);
+	get_background_color (style, GTK_STATE_FLAG_NORMAL, &back_color);
 
 	g_object_get (XPAD_TEXT_VIEW (pad->priv->textview), "follow-font-style", &follow_font_style, "follow-color-style", &follow_color_style, NULL);
 
@@ -1544,8 +1527,6 @@ menu_title_compare (GtkWindow *a, GtkWindow *b)
 	if (image) {\
 		item = gtk_menu_item_new ();\
 		GtkWidget *hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 3);\
-		GdkRGBA bg = {0, 0, 0, 0};\
-		gtk_widget_override_background_color (hbox, GTK_STATE_FLAG_NORMAL, &bg);\
 		gtk_container_add (GTK_CONTAINER (hbox), gtk_image_new_from_icon_name (image, GTK_ICON_SIZE_MENU));\
 		gtk_container_add (GTK_CONTAINER (hbox), gtk_label_new_with_mnemonic (mnemonic));\
 		gtk_container_add (GTK_CONTAINER (item), hbox);\
@@ -1581,6 +1562,7 @@ menu_get_popup_no_highlight (XpadPad *pad, GtkAccelGroup *accel_group)
 	menu = uppermenu;
 	MENU_ADD (_("_New"), "document-new", GDK_KEY_N, GDK_CONTROL_MASK, xpad_pad_spawn);
 	MENU_ADD (_("_Delete"), "edit-delete", GDK_KEY_Delete, GDK_SHIFT_MASK, xpad_pad_delete);
+	MENU_ADD (_("_Reload"), "reload-pad-content", GDK_KEY_F5, 0, xpad_pad_load_content);
 	MENU_ADD (_("_Close"), "window-close", 0, 0, xpad_pad_close);
 
 	/* Edit submenu */
@@ -1796,10 +1778,9 @@ xpad_pad_popup (XpadPad *pad, GdkEventButton *event)
 
 	menu_popup (pad);
 
-	if (event)
-		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, event->button, event->time);
-	else
-		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time ());
+	if (event) {
+		gtk_menu_popup_at_pointer (GTK_MENU (menu), (GdkEvent*) event);
+	}
 }
 
 /* These functions below are used to reduce the amounts of writes, hence improve the performance. */
@@ -1808,11 +1789,13 @@ void xpad_pad_save_content_delayed (XpadPad *pad)
 	pad->priv->unsaved_content = TRUE;
 	xpad_periodic_save_content_delayed (pad);
 }
+
 void xpad_pad_save_info_delayed (XpadPad *pad)
 {
 	pad->priv->unsaved_info = TRUE;
 	xpad_periodic_save_info_delayed (pad);
 }
+
 void xpad_pad_save_unsaved (XpadPad *pad)
 {
 	if (pad->priv->unsaved_content)
@@ -1820,3 +1803,18 @@ void xpad_pad_save_unsaved (XpadPad *pad)
 	if (pad->priv->unsaved_info)
 		xpad_pad_save_info (pad);
 }
+
+void xpad_pad_remove_accelerator_group (XpadPad *pad) {
+	g_return_if_fail (pad);
+
+	gtk_widget_add_events (GTK_WIDGET (pad), 0);
+
+	if (pad->priv->toolbar) {
+		gtk_widget_add_events (pad->priv->toolbar, 0);
+	}
+
+	if (pad->priv->accel_group) {
+		gtk_window_remove_accel_group (GTK_WINDOW(pad), pad->priv->accel_group);
+		g_clear_object (&pad->priv->accel_group);
+	}
+}
