@@ -26,6 +26,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #include "xpad-pad-group.h"
 #include "xpad-settings.h"
+#include "xpad-styling-helpers.h"
 
 G_BEGIN_DECLS
 
@@ -73,6 +74,7 @@ void xpad_pad_notify_clipboard_owner_changed (XpadPad *pad);
 void xpad_pad_notify_undo_redo_changed (XpadPad *pad);
 
 void xpad_pad_append_pad_titles_to_menu (GtkWidget *menu);
+void xpad_pad_remove_accelerator_group (XpadPad *pad);
 
 G_END_DECLS
 
