@@ -27,7 +27,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-struct XpadPreferencesPrivate 
+struct XpadPreferencesPrivate
 {
 	XpadSettings *settings;
 
@@ -52,6 +52,8 @@ struct XpadPreferencesPrivate
 	GtkWidget *editcheck;
 	GtkWidget *confirmcheck;
 	GtkWidget *has_decorations;
+	GtkWidget *hide_from_taskbar;
+	GtkWidget *hide_from_task_switcher;
 	GtkWidget *has_toolbar;
 	GtkWidget *autohide_toolbar;
 	GtkWidget *has_scrollbar;
@@ -72,6 +74,8 @@ struct XpadPreferencesPrivate
 	gulong editcheck_handler;
 	gulong confirmcheck_handler;
 	gulong has_decorations_handler;
+	gulong hide_from_taskbar_handler;
+	gulong hide_from_task_switcher_handler;
 	gulong has_toolbar_handler;
 	gulong autohide_toolbar_handler;
 	gulong has_scrollbar_handler;
@@ -90,18 +94,19 @@ struct XpadPreferencesPrivate
 	gulong notify_edit_handler;
 	gulong notify_confirm_handler;
 	gulong notify_has_decorations_handler;
+	gulong notify_hide_from_taskbar_handler;
+	gulong notify_hide_from_task_switcher_handler;
 	gulong notify_has_toolbar_handler;
 	gulong notify_autohide_toolbar_handler;
 	gulong notify_has_scrollbar_handler;
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (XpadPreferences, xpad_preferences, GTK_TYPE_DIALOG)
+G_DEFINE_TYPE_WITH_PRIVATE (XpadPreferences, xpad_preferences, GTK_TYPE_WINDOW)
 
 static void xpad_preferences_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void xpad_preferences_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 static void xpad_preferences_constructed (GObject *object);
 static void xpad_preferences_finalize (GObject *object);
-static void xpad_preferences_response (GtkDialog *dialog, gint response);
 
 static void change_font_check (GtkToggleButton *button, XpadPreferences *pref);
 static void change_font_face (GtkFontButton *button, XpadPreferences *pref);
@@ -119,6 +124,8 @@ static void change_tray_click (GtkComboBox *box, XpadPreferences *pref);
 static void change_edit_check (GtkToggleButton *button, XpadPreferences *pref);
 static void change_confirm_check (GtkToggleButton *button, XpadPreferences *pref);
 static void change_has_decorations (GtkToggleButton *button, XpadPreferences *pref);
+static void change_hide_from_taskbar (GtkToggleButton *button, XpadPreferences *pref);
+static void change_hide_from_task_switcher (GtkToggleButton *button, XpadPreferences *pref);
 static void change_has_toolbar (GtkToggleButton *button, XpadPreferences *pref);
 static void change_autohide_toolbar (GtkToggleButton *button, XpadPreferences *pref);
 static void change_has_scrollbar (GtkToggleButton *button, XpadPreferences *pref);
@@ -136,10 +143,12 @@ static void notify_tray_enabled (XpadPreferences *pref);
 static void notify_tray_click (XpadPreferences *pref);
 static void notify_edit (XpadPreferences *pref);
 static void notify_confirm (XpadPreferences *pref);
-static void notify_has_decorations(XpadPreferences *pref);
-static void notify_has_toolbar(XpadPreferences *pref);
-static void notify_autohide_toolbar(XpadPreferences *pref);
-static void notify_has_scrollbar(XpadPreferences *pref);
+static void notify_has_decorations (XpadPreferences *pref);
+static void notify_hide_from_taskbar (XpadPreferences *pref);
+static void notify_hide_from_task_switcher (XpadPreferences *pref);
+static void notify_has_toolbar (XpadPreferences *pref);
+static void notify_autohide_toolbar (XpadPreferences *pref);
+static void notify_has_scrollbar (XpadPreferences *pref);
 
 static GtkWidget * create_label (const gchar *label_text);
 
@@ -191,16 +200,15 @@ static void xpad_preferences_constructed (GObject *object)
 {
 	XpadPreferences *pref = XPAD_PREFERENCES (object);
 
-	GtkWidget *view_frame, *appearance_frame, *start_frame, *tray_frame, *other_frame, *label, *alignment;
-	GtkBox *font_hbox, *vbox, *hbox, *view_vbox, *appearance_vbox, *autostart_vbox, *tray_vbox, *other_vbox;
+	GtkWidget *label;
+	GtkBox *font_hbox, *vbox, *hbox, *view_vbox, *appearance_vbox, *start_vbox, *tray_vbox, *other_vbox;
 	const GdkRGBA *text_color, *back_color;
 	const gchar *fontname;
 	GtkStyleContext *style;
 	GtkSizeGroup *size_group_labels = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-	GtkRequisition req;
 	GdkRGBA theme_text_color = {0, 0, 0, 0}, theme_background_color = {0, 0, 0, 0};
 	guint tray_click_configuration, autostart_delay, autostart_display_pads;
-	gboolean confirm_destroy, edit_lock, autostart_xpad, autostart_wait_systray, autostart_new_pad, autostart_sticky, has_decorations, has_toolbar, autohide_toolbar, has_scrollbar;
+	gboolean confirm_destroy, edit_lock, autostart_xpad, autostart_wait_systray, autostart_new_pad, autostart_sticky, has_decorations, hide_from_taskbar, hide_from_task_switcher, has_toolbar, autohide_toolbar, has_scrollbar;
 
 	g_object_get (pref->priv->settings,
 			"fontname", &fontname,
@@ -216,6 +224,8 @@ static void xpad_preferences_constructed (GObject *object)
 			"autostart-delay", &autostart_delay,
 			"autostart-display-pads", &autostart_display_pads,
 			"has-decorations", &has_decorations,
+			"hide-from-taskbar", &hide_from_taskbar,
+			"hide-from-task-switcher", &hide_from_task_switcher,
 			"has-toolbar", &has_toolbar,
 			"autohide-toolbar", &autohide_toolbar,
 			"has-scrollbar", &has_scrollbar,
@@ -229,20 +239,12 @@ static void xpad_preferences_constructed (GObject *object)
 
 	view_vbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_VERTICAL, 20));
 	gtk_box_set_homogeneous (view_vbox, FALSE);
+        gtk_widget_set_margin_top (GTK_WIDGET (view_vbox), 12);
+        gtk_widget_set_margin_bottom (GTK_WIDGET (view_vbox), 12);
+        gtk_widget_set_margin_start (GTK_WIDGET (view_vbox), 12);
+        gtk_widget_set_margin_end (GTK_WIDGET (view_vbox), 12);
 
-	alignment = gtk_alignment_new (1, 1, 1, 1);
-	g_object_set (G_OBJECT (alignment),
-		"left-padding", 12,
-		"top-padding", 12,
-		"child", view_vbox,
-		NULL);
-	view_frame = GTK_WIDGET (g_object_new (GTK_TYPE_FRAME,
-		"label-widget", NULL,
-		"shadow-type", GTK_SHADOW_NONE,
-		"child", alignment,
-		NULL));
-
-	gtk_notebook_append_page (GTK_NOTEBOOK (pref->priv->notebook), GTK_WIDGET (view_frame), label);
+	gtk_notebook_append_page (GTK_NOTEBOOK (pref->priv->notebook), GTK_WIDGET (view_vbox), label);
 
 	pref->priv->has_toolbar = gtk_check_button_new_with_mnemonic (_("_Show toolbar"));
 	gtk_box_pack_start (view_vbox, pref->priv->has_toolbar, FALSE, FALSE, 0);
@@ -250,7 +252,6 @@ static void xpad_preferences_constructed (GObject *object)
 
 	pref->priv->autohide_toolbar = gtk_check_button_new_with_mnemonic (_("_Autohide toolbar"));
 	hbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 20));
-	gtk_box_pack_start (hbox, gtk_alignment_new (1, 1, 1, 1), FALSE, FALSE, 0);
 	gtk_box_pack_start (hbox, pref->priv->autohide_toolbar, FALSE, FALSE, 0);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pref->priv->autohide_toolbar), autohide_toolbar);
 	gtk_widget_set_sensitive (pref->priv->autohide_toolbar, has_toolbar);
@@ -268,45 +269,45 @@ static void xpad_preferences_constructed (GObject *object)
 	gtk_box_pack_start (view_vbox, pref->priv->has_decorations, FALSE, FALSE, 0);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pref->priv->has_decorations), has_decorations);
 
+	pref->priv->hide_from_taskbar = gtk_check_button_new_with_mnemonic (_("_Hide all notes from the taskbar and possibly the task switcher"));
+	gtk_box_pack_start (view_vbox, pref->priv->hide_from_taskbar, FALSE, FALSE, 0);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pref->priv->hide_from_taskbar), hide_from_taskbar);
+
+	pref->priv->hide_from_task_switcher = gtk_check_button_new_with_mnemonic (_("_Hide all notes from the workspace switcher and possibly the task switcher"));
+	gtk_box_pack_start (view_vbox, pref->priv->hide_from_task_switcher, FALSE, FALSE, 0);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pref->priv->hide_from_task_switcher), hide_from_task_switcher);
+
 	/* Layout options */
 	label = create_label (_("Layout"));
 
 	appearance_vbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_VERTICAL, 20));
 	gtk_box_set_homogeneous (appearance_vbox, FALSE);
+        gtk_widget_set_margin_top (GTK_WIDGET (appearance_vbox), 12);
+        gtk_widget_set_margin_bottom (GTK_WIDGET (appearance_vbox), 12);
+        gtk_widget_set_margin_start (GTK_WIDGET (appearance_vbox), 12);
+        gtk_widget_set_margin_end (GTK_WIDGET (appearance_vbox), 12);
 
-	alignment = gtk_alignment_new (1, 1, 1, 1);
-	g_object_set (G_OBJECT (alignment),
-		"left-padding", 12,
-		"top-padding", 12,
-		"child", appearance_vbox,
-		NULL);
-	appearance_frame = GTK_WIDGET (g_object_new (GTK_TYPE_FRAME,
-		"label-widget", NULL,
-		"shadow-type", GTK_SHADOW_NONE,
-		"child", alignment,
-		NULL));
-
-	gtk_notebook_append_page (GTK_NOTEBOOK (pref->priv->notebook), GTK_WIDGET (appearance_frame), label);
+	gtk_notebook_append_page (GTK_NOTEBOOK (pref->priv->notebook), GTK_WIDGET (appearance_vbox), label);
 
 	pref->priv->textbutton = gtk_color_button_new ();
 	pref->priv->fontbutton = gtk_font_button_new ();
 	pref->priv->backbutton = gtk_color_button_new ();
+	pref->priv->colorbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 
 	pref->priv->antifontcheck = gtk_radio_button_new_with_mnemonic (NULL, _("Use font from theme"));
 	pref->priv->fontcheck = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (pref->priv->antifontcheck), _("Use this font:"));
 	pref->priv->anticolorcheck = gtk_radio_button_new_with_mnemonic (NULL, _("Use colors from theme"));
 	pref->priv->colorcheck = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (pref->priv->anticolorcheck), _("Use these colors:"));
+        gtk_widget_set_margin_start (GTK_WIDGET (pref->priv->colorbox), 25);
 
 	font_hbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6));
 
 	gtk_box_pack_start (font_hbox, pref->priv->fontcheck, FALSE, TRUE, 0);
 	gtk_box_pack_start (font_hbox, pref->priv->fontbutton, FALSE, TRUE, 0);
 
-	pref->priv->colorbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 	hbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12));
 
 	label = gtk_label_new_with_mnemonic (_("Text:"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_size_group_add_widget (size_group_labels, label);
 	gtk_box_pack_start (hbox, label, FALSE, FALSE, 0);
 	gtk_box_pack_start (hbox, pref->priv->textbutton, FALSE, TRUE, 0);
@@ -315,7 +316,6 @@ static void xpad_preferences_constructed (GObject *object)
 	hbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12));
 
 	label = gtk_label_new_with_mnemonic (_("Background:"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_size_group_add_widget (size_group_labels, label);
 	gtk_box_pack_start (hbox, label, FALSE, FALSE, 0);
 	gtk_box_pack_start (hbox, pref->priv->backbutton, FALSE, TRUE, 0);
@@ -323,7 +323,7 @@ static void xpad_preferences_constructed (GObject *object)
 
 	style = gtk_widget_get_style_context (GTK_WIDGET(pref));
 	gtk_style_context_get_color (style, GTK_STATE_FLAG_NORMAL, &theme_text_color);
-	gtk_style_context_get_background_color (style, GTK_STATE_FLAG_NORMAL, &theme_background_color);
+	get_background_color (style, GTK_STATE_FLAG_NORMAL, &theme_background_color);
 
 	if (fontname)
 	{
@@ -359,10 +359,6 @@ static void xpad_preferences_constructed (GObject *object)
 		gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (pref->priv->backbutton), &theme_background_color);
 	}
 
-	alignment = gtk_alignment_new (1, 1, 1, 1);
-	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, 12, 0);
-	gtk_container_add (GTK_CONTAINER (alignment), pref->priv->colorbox);
-
 	vbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_VERTICAL, 6));
 
 	gtk_box_pack_start (vbox, pref->priv->antifontcheck, FALSE, FALSE, 0);
@@ -373,7 +369,7 @@ static void xpad_preferences_constructed (GObject *object)
 
 	gtk_box_pack_start (vbox, pref->priv->anticolorcheck, FALSE, FALSE, 0);
 	gtk_box_pack_start (vbox, pref->priv->colorcheck, FALSE, FALSE, 0);
-	gtk_box_pack_start (vbox, alignment, FALSE, FALSE, 0);
+	gtk_box_pack_start (vbox, pref->priv->colorbox, FALSE, FALSE, 0);
 	gtk_box_pack_start (appearance_vbox, GTK_WIDGET (vbox), FALSE, FALSE, 0);
 
 	gtk_color_chooser_set_use_alpha (GTK_COLOR_CHOOSER (pref->priv->textbutton), FALSE);
@@ -386,30 +382,21 @@ static void xpad_preferences_constructed (GObject *object)
 	/* Start options */
 	label = create_label (_("Startup"));
 
-	autostart_vbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_VERTICAL, 6));
-	gtk_box_set_homogeneous (autostart_vbox, FALSE);
+	start_vbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_VERTICAL, 6));
+	gtk_box_set_homogeneous (start_vbox, FALSE);
+        gtk_widget_set_margin_top (GTK_WIDGET (start_vbox), 12);
+        gtk_widget_set_margin_bottom (GTK_WIDGET (start_vbox), 12);
+        gtk_widget_set_margin_start (GTK_WIDGET (start_vbox), 12);
+        gtk_widget_set_margin_end (GTK_WIDGET (start_vbox), 12);
 
-	alignment = gtk_alignment_new (1, 1, 1, 1);
-	g_object_set (G_OBJECT (alignment),
-		"left-padding", 12,
-		"top-padding", 12,
-		"child", autostart_vbox,
-		NULL);
-	start_frame = GTK_WIDGET (g_object_new (GTK_TYPE_FRAME,
-		"label-widget", NULL,
-		"shadow-type", GTK_SHADOW_NONE,
-		"child", alignment,
-		NULL));
-
-	gtk_notebook_append_page (GTK_NOTEBOOK (pref->priv->notebook), GTK_WIDGET (start_frame), label);
+	gtk_notebook_append_page (GTK_NOTEBOOK (pref->priv->notebook), GTK_WIDGET (start_vbox), label);
 
 	pref->priv->autostart_xpad = gtk_check_button_new_with_mnemonic (_("_Start Xpad automatically after login"));
-	gtk_box_pack_start (autostart_vbox, pref->priv->autostart_xpad, FALSE, FALSE, 0);
+	gtk_box_pack_start (start_vbox, pref->priv->autostart_xpad, FALSE, FALSE, 0);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pref->priv->autostart_xpad), autostart_xpad);
 
 	pref->priv->autostart_wait_systray = gtk_check_button_new_with_mnemonic (_("_Wait for systray (if possible)"));
 	hbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 20));
-	gtk_box_pack_start (hbox, gtk_alignment_new (1, 1, 1, 1), FALSE, FALSE, 0);
 	gtk_box_pack_start (hbox, pref->priv->autostart_wait_systray, FALSE, FALSE, 0);
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pref->priv->autostart_wait_systray), autostart_wait_systray);
@@ -417,10 +404,10 @@ static void xpad_preferences_constructed (GObject *object)
 		gtk_widget_set_sensitive (pref->priv->autostart_wait_systray, TRUE);
 	else
 		gtk_widget_set_sensitive (pref->priv->autostart_wait_systray, FALSE);
-	gtk_box_pack_start (autostart_vbox, GTK_WIDGET (hbox), FALSE, FALSE, 0);
+	gtk_box_pack_start (start_vbox, GTK_WIDGET (hbox), FALSE, FALSE, 0);
 
 	pref->priv->autostart_new_pad = gtk_check_button_new_with_mnemonic (_("_Open a new empty pad"));
-	gtk_box_pack_start (autostart_vbox, pref->priv->autostart_new_pad, FALSE, FALSE, 0);
+	gtk_box_pack_start (start_vbox, pref->priv->autostart_new_pad, FALSE, FALSE, 0);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pref->priv->autostart_new_pad), autostart_new_pad);
 
 	label = gtk_label_new (_("Delay in seconds"));
@@ -432,7 +419,7 @@ static void xpad_preferences_constructed (GObject *object)
 	hbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12));
 	gtk_box_pack_start (hbox, label, FALSE, FALSE, 0);
 	gtk_box_pack_start (hbox, pref->priv->autostart_delay, FALSE, TRUE, 0);
-	gtk_box_pack_start (autostart_vbox, GTK_WIDGET (hbox), FALSE, FALSE, 0);
+	gtk_box_pack_start (start_vbox, GTK_WIDGET (hbox), FALSE, FALSE, 0);
 
 	label = gtk_label_new_with_mnemonic(_("Display pads"));
 	pref->priv->autostart_display_pads = gtk_combo_box_text_new();
@@ -443,27 +430,19 @@ static void xpad_preferences_constructed (GObject *object)
 	hbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12));
 	gtk_box_pack_start (hbox, label, FALSE, FALSE, 0);
 	gtk_box_pack_start (hbox, pref->priv->autostart_display_pads, FALSE, TRUE, 0);
-	gtk_box_pack_start (autostart_vbox, GTK_WIDGET (hbox), FALSE, FALSE, 0);
+	gtk_box_pack_start (start_vbox, GTK_WIDGET (hbox), FALSE, FALSE, 0);
 
 	/* Tray options */
 	label = create_label (_("Tray"));
 
 	tray_vbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_VERTICAL, 6));
 	gtk_box_set_homogeneous (tray_vbox, FALSE);
+        gtk_widget_set_margin_top (GTK_WIDGET (tray_vbox), 12);
+        gtk_widget_set_margin_bottom (GTK_WIDGET (tray_vbox), 12);
+        gtk_widget_set_margin_start (GTK_WIDGET (tray_vbox), 12);
+        gtk_widget_set_margin_end (GTK_WIDGET (tray_vbox), 12);
 
-	alignment = gtk_alignment_new (1, 1, 1, 1);
-	g_object_set (G_OBJECT (alignment),
-		"left-padding", 12,
-		"top-padding", 12,
-		"child", tray_vbox,
-		NULL);
-	tray_frame = GTK_WIDGET (g_object_new (GTK_TYPE_FRAME,
-		"label-widget", NULL,
-		"shadow-type", GTK_SHADOW_NONE,
-		"child", alignment,
-		NULL));
-
-	gtk_notebook_append_page (GTK_NOTEBOOK (pref->priv->notebook), GTK_WIDGET (tray_frame), label);
+	gtk_notebook_append_page (GTK_NOTEBOOK (pref->priv->notebook), GTK_WIDGET (tray_vbox), label);
 
 	pref->priv->tray_enabled = gtk_check_button_new_with_mnemonic (_("_Enable tray icon"));
 	gtk_box_pack_start (tray_vbox, pref->priv->tray_enabled, FALSE, FALSE, 0);
@@ -485,20 +464,12 @@ static void xpad_preferences_constructed (GObject *object)
 
 	other_vbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_VERTICAL, 6));
 	gtk_box_set_homogeneous (other_vbox, FALSE);
+        gtk_widget_set_margin_top (GTK_WIDGET (other_vbox), 12);
+        gtk_widget_set_margin_bottom (GTK_WIDGET (other_vbox), 12);
+        gtk_widget_set_margin_start (GTK_WIDGET (other_vbox), 12);
+        gtk_widget_set_margin_end (GTK_WIDGET (other_vbox), 12);
 
-	alignment = gtk_alignment_new (1, 1, 1, 1);
-	g_object_set (G_OBJECT (alignment),
-		"left-padding", 12,
-		"top-padding", 12,
-		"child", other_vbox,
-		NULL);
-	other_frame = GTK_WIDGET (g_object_new (GTK_TYPE_FRAME,
-		"label-widget", NULL,
-		"shadow-type", GTK_SHADOW_NONE,
-		"child", alignment,
-		NULL));
-
-	gtk_notebook_append_page (GTK_NOTEBOOK (pref->priv->notebook), GTK_WIDGET (other_frame), label);
+	gtk_notebook_append_page (GTK_NOTEBOOK (pref->priv->notebook), GTK_WIDGET (other_vbox), label);
 
 	pref->priv->editcheck = gtk_check_button_new_with_mnemonic (_("_Make pads read-only"));
 	pref->priv->confirmcheck = gtk_check_button_new_with_mnemonic (_("_Confirm pad deletion"));
@@ -510,13 +481,21 @@ static void xpad_preferences_constructed (GObject *object)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pref->priv->confirmcheck), confirm_destroy);
 
 	/* Close button and window title */
-	gtk_dialog_add_button (GTK_DIALOG (pref), "gtk-close", GTK_RESPONSE_CLOSE);
-	gtk_dialog_set_default_response (GTK_DIALOG (pref), GTK_RESPONSE_CLOSE);
-	g_signal_connect (pref, "response", G_CALLBACK (xpad_preferences_response), NULL);
+	GtkWidget *button = gtk_button_new_from_icon_name ("gtk-close", GTK_ICON_SIZE_BUTTON);
+	gtk_button_set_label (GTK_BUTTON (button), _("Close"));
+	g_signal_connect_swapped (GTK_BUTTON (button), "clicked", G_CALLBACK (gtk_widget_destroy), GTK_WIDGET (pref));
 	gtk_window_set_title (GTK_WINDOW (pref), _("Xpad Preferences"));
 
+	/* Add preference tabs and the close button together */
+	vbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_VERTICAL, 0));
+ 	gtk_container_add (GTK_CONTAINER (vbox), pref->priv->notebook);
+ 	gtk_container_add (GTK_CONTAINER (vbox), button);
+ 	gtk_container_add (GTK_CONTAINER (pref), GTK_WIDGET (vbox));
+
 	/* Activate all handlers */
 	pref->priv->has_decorations_handler = g_signal_connect (pref->priv->has_decorations, "toggled", G_CALLBACK (change_has_decorations), pref);
+	pref->priv->hide_from_taskbar_handler = g_signal_connect (pref->priv->hide_from_taskbar, "toggled", G_CALLBACK (change_hide_from_taskbar), pref);
+	pref->priv->hide_from_task_switcher_handler = g_signal_connect (pref->priv->hide_from_task_switcher, "toggled", G_CALLBACK (change_hide_from_task_switcher), pref);
 	pref->priv->has_toolbar_handler = g_signal_connect (pref->priv->has_toolbar, "toggled", G_CALLBACK (change_has_toolbar), pref);
 	pref->priv->autohide_toolbar_handler = g_signal_connect (pref->priv->autohide_toolbar, "toggled", G_CALLBACK (change_autohide_toolbar), pref);
 	pref->priv->has_scrollbar_handler = g_signal_connect (pref->priv->has_scrollbar, "toggled", G_CALLBACK (change_has_scrollbar), pref);
@@ -540,6 +519,8 @@ static void xpad_preferences_constructed (GObject *object)
 	pref->priv->confirmcheck_handler = g_signal_connect (pref->priv->confirmcheck, "toggled", G_CALLBACK (change_confirm_check), pref);
 
 	pref->priv->notify_has_decorations_handler = g_signal_connect_swapped (pref->priv->settings, "notify::has-decorations", G_CALLBACK (notify_has_decorations), pref);
+	pref->priv->notify_hide_from_taskbar_handler = g_signal_connect_swapped (pref->priv->settings, "notify::hide-from-taskbar", G_CALLBACK (notify_hide_from_taskbar), pref);
+	pref->priv->notify_hide_from_task_switcher_handler = g_signal_connect_swapped (pref->priv->settings, "notify::hide-from-task-switcher", G_CALLBACK (notify_hide_from_task_switcher), pref);
 	pref->priv->notify_has_toolbar_handler = g_signal_connect_swapped (pref->priv->settings, "notify::has-toolbar", G_CALLBACK (notify_has_toolbar), pref);
 	pref->priv->notify_autohide_toolbar_handler = g_signal_connect_swapped (pref->priv->settings, "notify::autohide-toolbar", G_CALLBACK (notify_autohide_toolbar), pref);
 	pref->priv->notify_has_scrollbar_handler = g_signal_connect_swapped (pref->priv->settings, "notify::has-scrollbar", G_CALLBACK (notify_has_scrollbar), pref);
@@ -559,19 +540,15 @@ static void xpad_preferences_constructed (GObject *object)
 	pref->priv->notify_tray_enabled_handler = g_signal_connect_swapped (pref->priv->settings, "notify::tray-enabled", G_CALLBACK (notify_tray_enabled), pref);
 	pref->priv->notify_tray_click_handler = g_signal_connect_swapped (pref->priv->settings, "notify::tray-click-configuration", G_CALLBACK(notify_tray_click), pref);
 
-	g_object_unref (size_group_labels);
+	g_clear_object (&size_group_labels);
 
-	/* Initiliaze the GUI logic */
+	/* Initialize the GUI logic */
 	g_object_notify (G_OBJECT (pref->priv->settings), "tray-enabled");
 
-	/* Make the preference dialog visible */
-	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (pref))), GTK_WIDGET (pref->priv->notebook), FALSE, FALSE, 0);
-	gtk_widget_show_all (gtk_dialog_get_content_area (GTK_DIALOG (pref)));
+	/* Make the preference window visible */
+	gtk_window_set_position (GTK_WINDOW (pref), GTK_WIN_POS_CENTER);
+	gtk_widget_show_all (GTK_WIDGET (pref));
 	gtk_window_set_resizable (GTK_WINDOW (pref), FALSE);
-
-	/* Make the preference window not so squished */
-	gtk_widget_get_preferred_size (GTK_WIDGET (pref), &req, NULL);
-	g_object_set (G_OBJECT (pref), "default-width", (gint) (req.height * 0.8), NULL);
 }
 
 static GtkWidget * create_label (const gchar *label_text) {
@@ -629,21 +606,17 @@ xpad_preferences_finalize (GObject *object)
 	G_OBJECT_CLASS (xpad_preferences_parent_class)->finalize (object);
 }
 
-static void
-xpad_preferences_response (GtkDialog *dialog, gint response)
-{
-	if (response == GTK_RESPONSE_CLOSE)
-		gtk_widget_destroy (GTK_WIDGET (dialog));
-}
-
 static void
 change_font_check (GtkToggleButton *button, XpadPreferences *pref)
 {
 	g_signal_handler_block (pref->priv->settings, pref->priv->notify_font_handler);
-	if (!gtk_toggle_button_get_active (button))
-		g_object_set (pref->priv->settings, "fontname", NULL, NULL);
-	else
+
+	if (gtk_toggle_button_get_active (button)) {
 		g_object_set (pref->priv->settings, "fontname", gtk_font_button_get_font_name (GTK_FONT_BUTTON (pref->priv->fontbutton)), NULL);
+	} else {
+		g_object_set (pref->priv->settings, "fontname", NULL, NULL);
+	}
+
 	gtk_widget_set_sensitive (pref->priv->fontbutton, gtk_toggle_button_get_active (button));
 	g_signal_handler_unblock (pref->priv->settings, pref->priv->notify_font_handler);
 }
@@ -662,14 +635,13 @@ change_color_check (GtkToggleButton *button, XpadPreferences *pref)
 	g_signal_handler_block (pref->priv->settings, pref->priv->notify_text_handler);
 	g_signal_handler_block (pref->priv->settings, pref->priv->notify_back_handler);
 
-	if (!gtk_toggle_button_get_active (button))
-		g_object_set (pref->priv->settings, "text-color", NULL, "back-color", NULL, NULL);
-	else
-	{
+	if (gtk_toggle_button_get_active (button)) {
 		GdkRGBA text_color, back_color;
 		gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (pref->priv->textbutton), &text_color);
 		gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (pref->priv->backbutton), &back_color);
 		g_object_set (pref->priv->settings, "text-color", &text_color, "back-color", &back_color, NULL);
+	} else {
+		g_object_set (pref->priv->settings, "text-color", NULL, "back-color", NULL, NULL);
 	}
 
 	gtk_widget_set_sensitive (pref->priv->colorbox, gtk_toggle_button_get_active (button));
@@ -740,7 +712,7 @@ change_autostart_wait_systray (GtkToggleButton *button, XpadPreferences *pref)
 		errtext = g_strdup_printf (_("Could not save %s\n%s"), filename, error->message);
 		xpad_app_error (NULL, _("Error changing wait for systray setting"), errtext);
 		g_free (errtext);
-		
+
 		gtk_toggle_button_set_active (button, !wait_systray);
 		return;
 	}
@@ -820,6 +792,22 @@ change_has_decorations (GtkToggleButton *button, XpadPreferences *pref)
 	g_signal_handler_unblock (pref->priv->settings, pref->priv->notify_has_decorations_handler);
 }
 
+static void
+change_hide_from_taskbar (GtkToggleButton *button, XpadPreferences *pref)
+{
+	g_signal_handler_block (pref->priv->settings, pref->priv->notify_hide_from_taskbar_handler);
+	g_object_set (pref->priv->settings, "hide-from-taskbar", gtk_toggle_button_get_active (button), NULL);
+	g_signal_handler_unblock (pref->priv->settings, pref->priv->notify_hide_from_taskbar_handler);
+}
+
+static void
+change_hide_from_task_switcher (GtkToggleButton *button, XpadPreferences *pref)
+{
+	g_signal_handler_block (pref->priv->settings, pref->priv->notify_hide_from_task_switcher_handler);
+	g_object_set (pref->priv->settings, "hide-from-task-switcher", gtk_toggle_button_get_active (button), NULL);
+	g_signal_handler_unblock (pref->priv->settings, pref->priv->notify_hide_from_task_switcher_handler);
+}
+
 static void
 change_has_toolbar (GtkToggleButton *button, XpadPreferences *pref)
 {
@@ -995,7 +983,7 @@ notify_tray_enabled (XpadPreferences *pref)
 	g_signal_handler_unblock (pref->priv->tray_enabled, pref->priv->tray_enabled_handler);
 }
 
-static void 
+static void
 notify_tray_click (XpadPreferences *pref)
 {
 	guint value;
@@ -1035,6 +1023,26 @@ notify_has_decorations (XpadPreferences *pref)
 	g_signal_handler_unblock (pref->priv->has_decorations, pref->priv->has_decorations_handler);
 }
 
+static void
+notify_hide_from_taskbar (XpadPreferences *pref)
+{
+	gboolean value;
+	g_object_get (pref->priv->settings, "hide-from-taskbar", &value, NULL);
+	g_signal_handler_block (pref->priv->hide_from_taskbar, pref->priv->hide_from_taskbar_handler);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pref->priv->hide_from_taskbar), value);
+	g_signal_handler_unblock (pref->priv->hide_from_taskbar, pref->priv->hide_from_taskbar_handler);
+}
+
+static void
+notify_hide_from_task_switcher (XpadPreferences *pref)
+{
+	gboolean value;
+	g_object_get (pref->priv->settings, "hide-from-task-switcher", &value, NULL);
+	g_signal_handler_block (pref->priv->hide_from_task_switcher, pref->priv->hide_from_task_switcher_handler);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pref->priv->hide_from_task_switcher), value);
+	g_signal_handler_unblock (pref->priv->hide_from_task_switcher, pref->priv->hide_from_task_switcher_handler);
+}
+
 static void
 notify_has_toolbar (XpadPreferences *pref)
 {
