@@ -21,9 +21,9 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #include "../config.h"
 
-#include "xpad-settings.h"
 #include "fio.h"
 #include "xpad-app.h"
+#include "xpad-settings.h"
 
 #include <string.h>
 
@@ -95,6 +95,11 @@ enum
 static GParamSpec *obj_prop[N_PROPERTIES] = { NULL, };
 static guint signals[LAST_SIGNAL] = { 0 };
 
+/* A pleasant light yellow background color, similar to commercial sticky notes, with black text. */
+static const GdkRGBA text_color_default = {0, 0, 0, 1};
+static const GdkRGBA back_color_default = {1, 0.933334350586, 0.6, 1};
+static const gchar *font_default = "Sans 9";
+
 static void load_from_file (XpadSettings *settings, const gchar *filename);
 static void save_to_file (XpadSettings *settings, const gchar *filename);
 static void xpad_settings_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
@@ -135,8 +140,7 @@ xpad_settings_class_init (XpadSettingsClass *klass)
 	obj_prop[PROP_AUTOSTART_STICKY] = g_param_spec_boolean ("autostart-sticky", "Stick to desktop", "Whether pads are sticky on creation", FALSE, G_PARAM_READWRITE);
 	obj_prop[PROP_AUTOSTART_DELAY] = g_param_spec_uint ("autostart-delay", "Delay autostart of Xpad", "Number of seconds to wait before start of Xpad", 0, G_MAXUINT, 0, G_PARAM_READWRITE);
 	obj_prop[PROP_AUTOSTART_DISPLAY_PADS] = g_param_spec_uint ("autostart-display-pads", "Autostart display pads", "Show/hide/restore pads at start", 0, G_MAXUINT, 2, G_PARAM_READWRITE);
-	obj_prop[PROP_HIDE_FROM_TASKBAR] = g_param_spec_boolean ("hide-from-taskbar", "Hide from taskbar", "Hide the pads from the task bar", FALSE, 
-G_PARAM_READWRITE);
+	obj_prop[PROP_HIDE_FROM_TASKBAR] = g_param_spec_boolean ("hide-from-taskbar", "Hide from taskbar", "Hide the pads from the task bar", FALSE, G_PARAM_READWRITE);
 	obj_prop[PROP_HIDE_FROM_TASK_SWITCHER] = g_param_spec_boolean ("hide-from-task-switcher", "Hide from task switcher", "Hide the pads from the task or workspace switcher", FALSE, G_PARAM_READWRITE);
 
 	g_object_class_install_properties (gobject_class, N_PROPERTIES, obj_prop);
@@ -170,10 +174,9 @@ xpad_settings_init (XpadSettings *settings)
 	settings->priv->has_toolbar = TRUE;
 	settings->priv->autohide_toolbar = TRUE;
 	settings->priv->has_scrollbar = TRUE;
-	settings->priv->fontname = NULL;
-	/* A pleasant light yellow background color, similar to commercial sticky notes, with black text. */
-	settings->priv->text = gdk_rgba_copy(&(GdkRGBA) {0, 0, 0, 1});
-	settings->priv->back = gdk_rgba_copy(&(GdkRGBA) {1, 0.933334350586, 0.6, 1});
+	settings->priv->fontname = g_strdup (font_default);
+	settings->priv->text = gdk_rgba_copy (&text_color_default);
+	settings->priv->back = gdk_rgba_copy (&back_color_default);
 	settings->priv->autostart_wait_systray = TRUE;
 	settings->priv->autostart_new_pad = FALSE;
 	settings->priv->autostart_sticky = FALSE;
@@ -318,21 +321,29 @@ xpad_settings_set_property (GObject *object, guint prop_id, const GValue *value,
 		break;
 
 	case PROP_TEXT_COLOR:
-		if (settings->priv->text)
+		if (settings->priv->text) {
 			gdk_rgba_free (settings->priv->text);
-		if (g_value_get_boxed (value))
+		}
+
+		if (g_value_get_boxed (value)) {
 			settings->priv->text = gdk_rgba_copy (g_value_get_boxed (value));
-		else
+		} else {
 			settings->priv->text = NULL;
+		}
+
 		break;
 
 	case PROP_BACK_COLOR:
-		if (settings->priv->back)
+		if (settings->priv->back) {
 			gdk_rgba_free (settings->priv->back);
-		if (g_value_get_boxed (value))
+		}
+
+		if (g_value_get_boxed (value)) {
 			settings->priv->back = gdk_rgba_copy (g_value_get_boxed (value));
-		else
+		} else {
 			settings->priv->back = NULL;
+		}
+
 		break;
 
 	case PROP_FONTNAME:
@@ -522,19 +533,11 @@ static void
 load_from_file (XpadSettings *settings, const gchar *filename)
 {
 	gchar *buttons = NULL, *text_color_string = NULL, *background_color_string = NULL;
-	GdkRGBA text_color = {0, 0, 0, 0}, back_color = {0, 0, 0, 0};
-	gboolean use_text, use_back;
-
-	use_text = settings->priv->text ? TRUE : FALSE;
-	if (settings->priv->text)
-		text_color = *settings->priv->text;
-
-	use_back = settings->priv->back ? TRUE : FALSE;
-	if (settings->priv->back)
-		back_color = *settings->priv->back;
+	GdkRGBA text_color = text_color_default, back_color = back_color_default;
+	gboolean use_text = TRUE, use_back = TRUE;
 
 	/* get all the values from the default-style text file in the forms of booleans, ints or strings. */
-	if (fio_get_values_from_file (filename, 
+	if (fio_get_values_from_file (filename,
 		"b|decorations", &settings->priv->has_decorations,
 		"u|height", &settings->priv->height,
 		"u|width", &settings->priv->width,
@@ -561,48 +564,52 @@ load_from_file (XpadSettings *settings, const gchar *filename)
 		NULL))
 		return;
 
-	if (use_text) {
+	/* Remove the existing value, since we are going to overwrite it */
+	if (settings->priv->text) {
 		gdk_rgba_free (settings->priv->text);
+	}
 
-		/*
-		 * If, for some reason, one of the colors could not be retrieved
-		 * (for example due to the migration to the new GdkRGBA colors),
-		 * set the color to the default.
-		 */
-		if (text_color_string == NULL)
-			text_color = (GdkRGBA) {0, 0, 0, 1};
-		else
-			/* If, for some reason, the parsing of the colors fail, set the color to the default. */
-			if (!gdk_rgba_parse (&text_color, text_color_string))
-				text_color = (GdkRGBA) {0, 0, 0, 1};
+	if (use_text) {
+		/* The user prefers a custom text color */
+		if (text_color_string != NULL) {
+			/* If parsing succeeds, then text_color is overwritten.
+			 * If it fails, the default remains.
+			 */
+			gdk_rgba_parse (&text_color, text_color_string);
+		}
 
 		settings->priv->text = gdk_rgba_copy (&text_color);
+	} else {
+		/* The user prefers the text color of the GTK theme */
+		settings->priv->text = NULL;
+	}
+
+	/* Remove the existing value, since we are going to overwrite it */
+	if (settings->priv->back) {
+		gdk_rgba_free (settings->priv->back);
 	}
 
-	gdk_rgba_free (settings->priv->back);
 	if (use_back) {
-		/*
-		 * If, for some reason, one of the colors could not be retrieved
-		 * (for example due to the migration to the new GdkRGBA colors),
-		 * set the color to the default.
-		 */
-		if (background_color_string == NULL)
-			back_color = (GdkRGBA) {1, 0.933334350586, 0.6, 1};
-		else
-			/* If, for some reason, the parsing of the colors fail, set the color to the default. */
-			if (!gdk_rgba_parse (&back_color, background_color_string))
-				back_color = (GdkRGBA) {1, 0.933334350586, 0.6, 1};
+		/* The user prefers a custom background color */
+		if (background_color_string != NULL) {
+			/* If parsing succeeds, then back_color is overwritten.
+			 * If it fails, the default remains.
+			 */
+			gdk_rgba_parse (&back_color, background_color_string);
+		}
 
 		settings->priv->back = gdk_rgba_copy (&back_color);
-	}
-	else
+	} else {
+		/* The user prefers the background color of the GTK theme*/
 		settings->priv->back = NULL;
+	}
 
-	if (settings->priv->fontname && strcmp (settings->priv->fontname, "NULL") == 0)
-		settings->priv->fontname = NULL;
+	/* If the String value of NULL has been retrieved, then the user wants to follow the GTK theme font */
+    if (settings->priv->fontname && strcmp (settings->priv->fontname, "NULL") == 0) {
+    	settings->priv->fontname = NULL;
+    }
 
-	if (buttons)
-	{
+	if (buttons) {
 		gint i;
 		gchar **button_names;
 
@@ -647,7 +654,7 @@ save_to_file (XpadSettings *settings, const gchar *filename)
 		tmp = tmp->next;
 	}
 
-	fio_set_values_to_file (filename, 
+	fio_set_values_to_file (filename,
 		"b|decorations", settings->priv->has_decorations,
 		"u|height", settings->priv->height,
 		"u|width", settings->priv->width,
