@@ -28,13 +28,14 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #include <gtk/gtk.h>
 
-struct XpadTextViewPrivate 
+struct XpadTextViewPrivate
 {
 	gboolean follow_font_style;
 	gboolean follow_color_style;
 	gulong notify_text_handler;
 	gulong notify_back_handler;
 	gulong notify_font_handler;
+	GtkCssProvider *font_provider;
 	XpadTextBuffer *buffer;
 	XpadSettings *settings;
 	XpadPad *pad;
@@ -54,6 +55,7 @@ static void xpad_text_view_notify_edit_lock (XpadTextView *view);
 static void xpad_text_view_notify_editable (XpadTextView *view);
 static void xpad_text_view_notify_fontname (XpadTextView *view);
 static void xpad_text_view_notify_colors (XpadTextView *view);
+gchar * pango_font_description_to_css (PangoFontDescription *desc);
 
 enum
 {
@@ -127,18 +129,10 @@ xpad_text_view_dispose (GObject *object)
 {
 	XpadTextView *view = XPAD_TEXT_VIEW (object);
 
-	if (view->priv->buffer)
-		g_object_unref (view->priv->buffer);
-
-	if (view->priv->pad) {
-		g_object_unref(view->priv->pad);
-		view->priv->pad = NULL;
-	}
-
-	if (view->priv->settings) {
-		g_object_unref(view->priv->settings);
-		view->priv->settings = NULL;
-	}
+	g_clear_object (&view->priv->buffer);
+	g_clear_object (&view->priv->pad);
+	g_clear_object (&view->priv->settings);
+	g_clear_object (&view->priv->font_provider);
 
 	G_OBJECT_CLASS (xpad_text_view_parent_class)->dispose (object);
 }
@@ -288,21 +282,22 @@ xpad_text_view_notify_editable (XpadTextView *view)
 {
 	GdkCursor *cursor;
 	gboolean editable;
+	GdkDisplay *display;
 	GdkWindow *view_window;
 	GtkSourceView *view_tv = GTK_SOURCE_VIEW (view);
 
 	editable = gtk_text_view_get_editable (GTK_TEXT_VIEW (view_tv));
 	gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (view_tv), editable);
 
-	cursor = editable ? gdk_cursor_new (GDK_XTERM) : NULL;
+	view_window = gtk_text_view_get_window (GTK_TEXT_VIEW (view_tv), GTK_TEXT_WINDOW_TEXT);
+	display = gdk_window_get_display(view_window);
+	cursor = editable ? gdk_cursor_new_for_display (display, GDK_XTERM) : NULL;
 
 	/* Only set for pads which are currently visible */ 
-	view_window = gtk_text_view_get_window (GTK_TEXT_VIEW (view_tv), GTK_TEXT_WINDOW_TEXT);
 	if (view_window != NULL)
 		gdk_window_set_cursor (view_window, cursor);
 
-	if (cursor)
-		g_object_unref (cursor);
+	g_clear_object (&cursor);
 }
 
 static void
@@ -310,8 +305,9 @@ xpad_text_view_notify_fontname (XpadTextView *view)
 {
 	const gchar *font;
 	g_object_get (view->priv->settings, "fontname", &font, NULL);
+
 	PangoFontDescription *fontdesc = font ? pango_font_description_from_string (font) : NULL;
-	gtk_widget_override_font (GTK_WIDGET (view), fontdesc);
+	xpad_text_view_set_font (GTK_WIDGET (view), fontdesc);
 	if (fontdesc)
 		pango_font_description_free (fontdesc);
 }
@@ -320,21 +316,68 @@ xpad_text_view_notify_fontname (XpadTextView *view)
 static void
 xpad_text_view_notify_colors (XpadTextView *view)
 {
-	GtkWidget *view_widget = GTK_WIDGET (view);
-	/* Set the colors of this individual pad to the global setting preference. */
-	const GdkRGBA *text_color, *back_color;
-
 	if (view->priv->follow_color_style) {
+		/* Set the colors of this individual pad to the global setting preference. */
+		const GdkRGBA *text_color, *back_color;
+
+		GtkWidget *view_widget = GTK_WIDGET (view);
+
 		/* Set the colors to the global preferences colors */
 		g_object_get (view->priv->settings, "text-color", &text_color, "back-color", &back_color, NULL);
+		xpad_text_view_set_colors(view_widget, text_color, back_color);
+	}
+}
 
-		gtk_widget_override_cursor (view_widget, text_color, text_color);
-		gtk_widget_override_color (view_widget, GTK_STATE_FLAG_NORMAL, text_color);
-		gtk_widget_override_background_color (view_widget, GTK_STATE_FLAG_NORMAL, back_color);
+/* Set the foreground and background color of the visible part of the pad, which is the text view */
+void xpad_text_view_set_colors (GtkWidget *view, const GdkRGBA *text_color, const GdkRGBA *back_color) {
+	gchar *cssStyling = g_strconcat("textview, textview text {caret-color: ",
+			text_color ? gdk_rgba_to_string (text_color) : "@theme-fg_color",
+			"; color: ",
+			text_color ? gdk_rgba_to_string (text_color) : "@theme_fg_color",
+			"; background-color: ",
+			back_color ? gdk_rgba_to_string (back_color) : "@theme_bg_color",
+			";}\n", NULL);
 
-		/* Inverse the text and background colors for selected text, so it is likely to be visible by any choice of the colors. */
-		gtk_widget_override_color (view_widget, GTK_STATE_FLAG_SELECTED, back_color);
-		gtk_widget_override_background_color (view_widget, GTK_STATE_FLAG_SELECTED, text_color);
+	/*
+	 * TODO: If the background color is close to the text selection background color (blue-ish),
+	 * then change the color for selected text by adding this to the CSS.
+	 * */
 
+	GtkStyleContext *context = gtk_widget_get_style_context (view);
+	GtkCssProvider *provider = gtk_css_provider_new ();
+	gtk_css_provider_load_from_data (provider, cssStyling, -1, NULL);
+	gtk_style_context_add_provider (context, GTK_STYLE_PROVIDER (provider), GTK_STYLE_PROVIDER_PRIORITY_SETTINGS);
+
+	g_free(cssStyling);
+	g_clear_object (&provider);
+}
+
+/* Set the font of the pad, which is in the text view */
+void xpad_text_view_set_font (GtkWidget *view, PangoFontDescription *desc) {
+	GtkStyleContext *context = gtk_widget_get_style_context (view);
+
+	if (desc == NULL && XPAD_TEXT_VIEW (view)->priv->font_provider) {
+		/* Remove font provider */
+		gtk_style_context_remove_provider (context, GTK_STYLE_PROVIDER (XPAD_TEXT_VIEW (view)->priv->font_provider));
+		g_clear_object (&XPAD_TEXT_VIEW (view)->priv->font_provider);
+	} else {
+		/* Add/replace font provider */
+		const gchar *font_description;
+		gchar *cssStyling;
+
+		font_description = pango_font_description_to_css(desc);
+		cssStyling = g_strconcat("textview, textview text ", font_description, "\n", NULL);
+
+		if (XPAD_TEXT_VIEW (view)->priv->font_provider) {
+			gtk_style_context_remove_provider (context, GTK_STYLE_PROVIDER (XPAD_TEXT_VIEW (view)->priv->font_provider));
+			g_clear_object (&XPAD_TEXT_VIEW (view)->priv->font_provider);
+		}
+
+		GtkCssProvider *provider = gtk_css_provider_new ();
+		gtk_css_provider_load_from_data (provider, cssStyling, -1, NULL);
+		gtk_style_context_add_provider (context, GTK_STYLE_PROVIDER (provider), GTK_STYLE_PROVIDER_PRIORITY_SETTINGS);
+		XPAD_TEXT_VIEW (view)->priv->font_provider = provider;
+
+		g_free (cssStyling);
 	}
 }
