@@ -1,6 +1,7 @@
 /*
 
 Copyright (c) 2001-2007 Michael Terry
+Copyright (c) 2013-2014 Arthur Borsboom
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,13 +20,13 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
 #include "../config.h"
-#include <string.h>
-#include <gdk/gdkkeysyms.h>
-#include <glib/gi18n.h>
+
 #include "xpad-toolbar.h"
-#include "xpad-settings.h"
+#include "xpad-app.h"
 #include "xpad-grip-tool-item.h"
 
+#include <glib/gi18n.h>
+
 struct XpadToolbarPrivate
 {
 	gboolean move_removed;
@@ -34,11 +35,9 @@ struct XpadToolbarPrivate
 	guint move_button_release_handler;
 	guint move_key_press_handler;
 	XpadPad *pad;
-	// GtkToolItem *move_button;
-	// GtkToolItem *tool_items;
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE(XpadToolbar, xpad_toolbar, GTK_TYPE_TOOLBAR)
+G_DEFINE_TYPE_WITH_PRIVATE (XpadToolbar, xpad_toolbar, GTK_TYPE_TOOLBAR)
 
 enum {
 	XPAD_BUTTON_TYPE_SEPARATOR,
@@ -49,7 +48,7 @@ enum {
 typedef struct
 {
 	const gchar *name;
-	const gchar *stock;
+	const gchar *icon_name;
 	guint signal;
 	guint type;
 	const gchar *desc;
@@ -77,45 +76,32 @@ enum
 
 static const XpadToolbarButton buttons[] =
 {
-	{"Clear", GTK_STOCK_CLEAR, ACTIVATE_CLEAR, XPAD_BUTTON_TYPE_BUTTON, N_("Clear Pad Contents"), N_("Add C_lear to Toolbar")},
-	{"Close", GTK_STOCK_CLOSE, ACTIVATE_CLOSE, XPAD_BUTTON_TYPE_BUTTON, N_("Close and Save Pad"), N_("Add _Close to Toolbar")},
-	{"Copy", GTK_STOCK_COPY, ACTIVATE_COPY, XPAD_BUTTON_TYPE_BUTTON, N_("Copy to Clipboard"), N_("Add C_opy to Toolbar")},
-	{"Cut", GTK_STOCK_CUT, ACTIVATE_CUT, XPAD_BUTTON_TYPE_BUTTON, N_("Cut to Clipboard"), N_("Add C_ut to Toolbar")},
-	{"Delete", GTK_STOCK_DELETE, ACTIVATE_DELETE, XPAD_BUTTON_TYPE_BUTTON, N_("Delete Pad"), N_("Add _Delete to Toolbar")},
-	{"New", GTK_STOCK_NEW, ACTIVATE_NEW, XPAD_BUTTON_TYPE_BUTTON, N_("Open New Pad"), N_("Add _New to Toolbar")},
-	{"Paste", GTK_STOCK_PASTE, ACTIVATE_PASTE, XPAD_BUTTON_TYPE_BUTTON, N_("Paste from Clipboard"), N_("Add Pa_ste to Toolbar")},
-	{"Preferences", GTK_STOCK_PREFERENCES, ACTIVATE_PREFERENCES, XPAD_BUTTON_TYPE_BUTTON, N_("Edit Preferences"), N_("Add Pr_eferences to Toolbar")},
-	{"Properties", GTK_STOCK_PROPERTIES, ACTIVATE_PROPERTIES, XPAD_BUTTON_TYPE_BUTTON, N_("Edit Pad Properties"), N_("Add Proper_ties to Toolbar")},
-	{"Redo", GTK_STOCK_REDO, ACTIVATE_REDO, XPAD_BUTTON_TYPE_BUTTON, N_("Redo"), N_("Add _Redo to Toolbar")},
-	{"Quit", GTK_STOCK_QUIT, ACTIVATE_QUIT, XPAD_BUTTON_TYPE_BUTTON, N_("Close All Pads"), N_("Add Close _All to Toolbar")},
-	{"Undo", GTK_STOCK_UNDO, ACTIVATE_UNDO, XPAD_BUTTON_TYPE_BUTTON, N_("Undo"), N_("Add _Undo to Toolbar")},
-	{"sep", NULL, 0, XPAD_BUTTON_TYPE_SEPARATOR, NULL, N_("Add a Se_parator to Toolbar")} /* Separator */
-	/*{"Minimize to Tray", "gtk-goto-bottom", 1, N_("Minimize Pads to System Tray")}*/
+	{"Clear", "edit-clear", ACTIVATE_CLEAR, XPAD_BUTTON_TYPE_BUTTON, N_("Clear Pad Contents"), N_("Add C_lear button")},
+	{"Close", "window-close", ACTIVATE_CLOSE, XPAD_BUTTON_TYPE_BUTTON, N_("Close and Save Pad"), N_("Add _Close button")},
+	{"Copy", "edit-copy", ACTIVATE_COPY, XPAD_BUTTON_TYPE_BUTTON, N_("Copy to Clipboard"), N_("Add C_opy button")},
+	{"Cut", "edit-cut", ACTIVATE_CUT, XPAD_BUTTON_TYPE_BUTTON, N_("Cut to Clipboard"), N_("Add C_ut button")},
+	{"Delete", "edit-delete", ACTIVATE_DELETE, XPAD_BUTTON_TYPE_BUTTON, N_("Delete Pad"), N_("Add _Delete button")},
+	{"New", "document-new", ACTIVATE_NEW, XPAD_BUTTON_TYPE_BUTTON, N_("Open New Pad"), N_("Add _New button")},
+	{"Paste", "edit-paste", ACTIVATE_PASTE, XPAD_BUTTON_TYPE_BUTTON, N_("Paste from Clipboard"), N_("Add Pa_ste button")},
+	{"Preferences", "preferences-system", ACTIVATE_PREFERENCES, XPAD_BUTTON_TYPE_BUTTON, N_("Edit Preferences"), N_("Add Pr_eferences button")},
+	{"Properties", "document-properties", ACTIVATE_PROPERTIES, XPAD_BUTTON_TYPE_BUTTON, N_("Edit Pad Properties"), N_("Add Proper_ties button")},
+	{"Redo", "edit-redo", ACTIVATE_REDO, XPAD_BUTTON_TYPE_BUTTON, N_("Redo"), N_("Add _Redo button")},
+	{"Quit", "application-exit", ACTIVATE_QUIT, XPAD_BUTTON_TYPE_BUTTON, N_("Close All Pads"), N_("Add Close _All button")},
+	{"Undo", "edit-undo", ACTIVATE_UNDO, XPAD_BUTTON_TYPE_BUTTON, N_("Undo"), N_("Add _Undo button")},
+	{"Separator", NULL, 0, XPAD_BUTTON_TYPE_SEPARATOR, NULL, N_("Add Se_parator")}
 };
 
-static G_CONST_RETURN XpadToolbarButton *xpad_toolbar_button_lookup (XpadToolbar *toolbar, const gchar *name);
+static void xpad_toolbar_constructed (GObject *object);
+static void xpad_toolbar_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
+static void xpad_toolbar_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
+static void xpad_toolbar_dispose (GObject *object);
+static const XpadToolbarButton *xpad_toolbar_button_lookup (XpadToolbar *toolbar, const gchar *name);
 static GtkToolItem *xpad_toolbar_button_to_item (XpadToolbar *toolbar, const XpadToolbarButton *button);
 static void xpad_toolbar_button_activated (GtkToolButton *button);
 static void xpad_toolbar_change_buttons (XpadToolbar *toolbar);
-static void xpad_toolbar_dispose (GObject *object);
-static void xpad_toolbar_finalize (GObject *object);
-static void xpad_toolbar_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
-static void xpad_toolbar_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
-static void xpad_toolbar_remove_all_buttons ();
-static void xpad_toolbar_remove_last_button ();
-static void xpad_toolbar_add_button (const gchar *button_name);
+static void xpad_toolbar_add_button (GtkMenuItem *menu_item, XpadSettings *settings);
 static gboolean xpad_toolbar_popup_context_menu (GtkToolbar *toolbar, gint x, gint y, gint button);
 
-/* Unfinished new feature of previous developer
-static void xpad_toolbar_remove_button (GtkWidget *button);
-static gboolean xpad_toolbar_button_press_event (GtkWidget *widget, GdkEventButton *event);
-static gboolean xpad_toolbar_popup_button_menu (GtkWidget *button, GdkEventButton *event, XpadToolbar *toolbar);
-static gboolean xpad_toolbar_move_button_start (XpadToolbar *toolbar, GtkWidget *button);
-static gboolean xpad_toolbar_move_button_move (XpadToolbar *toolbar, GdkEventMotion *event);
-static gboolean xpad_toolbar_move_button_move_keyboard (XpadToolbar *toolbar, GdkEventKey *event);
-static gboolean xpad_toolbar_move_button_end (XpadToolbar *toolbar);
-*/
-
 static guint signals[LAST_SIGNAL] = { 0 };
 
 enum
@@ -136,14 +122,13 @@ xpad_toolbar_class_init (XpadToolbarClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 	GtkToolbarClass *gtktoolbar_class = GTK_TOOLBAR_CLASS (klass);
-	
-	gtktoolbar_class->popup_context_menu = xpad_toolbar_popup_context_menu;
+
+	gobject_class->constructed = xpad_toolbar_constructed;
 	gobject_class->set_property = xpad_toolbar_set_property;
 	gobject_class->get_property = xpad_toolbar_get_property;
 	gobject_class->dispose = xpad_toolbar_dispose;
-	gobject_class->finalize = xpad_toolbar_finalize;
-	
-	/* Signals */
+	gtktoolbar_class->popup_context_menu = xpad_toolbar_popup_context_menu;
+
 	signals[ACTIVATE_NEW] = 
 		g_signal_new ("activate-new",
 		              G_OBJECT_CLASS_TYPE (gobject_class),
@@ -151,7 +136,7 @@ xpad_toolbar_class_init (XpadToolbarClass *klass)
 		              G_STRUCT_OFFSET (XpadToolbarClass, activate_new),
 		              NULL, NULL,
 		              g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
-	
+
 	signals[ACTIVATE_CLOSE] = 
 		g_signal_new ("activate-close",
 		              G_OBJECT_CLASS_TYPE (gobject_class),
@@ -199,7 +184,7 @@ xpad_toolbar_class_init (XpadToolbarClass *klass)
 		              G_STRUCT_OFFSET (XpadToolbarClass, activate_paste),
 		              NULL, NULL,
 		              g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
-	
+
 	signals[ACTIVATE_QUIT] = 
 		g_signal_new ("activate-quit",
 		              G_OBJECT_CLASS_TYPE (gobject_class),
@@ -207,7 +192,7 @@ xpad_toolbar_class_init (XpadToolbarClass *klass)
 		              G_STRUCT_OFFSET (XpadToolbarClass, activate_quit),
 		              NULL, NULL,
 		              g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
-	
+
 	signals[ACTIVATE_CLEAR] = 
 		g_signal_new ("activate-clear",
 		              G_OBJECT_CLASS_TYPE (gobject_class),
@@ -223,7 +208,7 @@ xpad_toolbar_class_init (XpadToolbarClass *klass)
 		              G_STRUCT_OFFSET (XpadToolbarClass, activate_properties),
 		              NULL, NULL,
 		              g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
-	
+
 	signals[ACTIVATE_PREFERENCES] = 
 		g_signal_new ("activate-preferences",
 		              G_OBJECT_CLASS_TYPE (gobject_class),
@@ -231,7 +216,7 @@ xpad_toolbar_class_init (XpadToolbarClass *klass)
 		              G_STRUCT_OFFSET (XpadToolbarClass, activate_preferences),
 		              NULL, NULL,
 		              g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
-	
+
 	signals[ACTIVATE_DELETE] = 
 		g_signal_new ("activate-delete",
 		              G_OBJECT_CLASS_TYPE (gobject_class),
@@ -239,7 +224,7 @@ xpad_toolbar_class_init (XpadToolbarClass *klass)
 		              G_STRUCT_OFFSET (XpadToolbarClass, activate_delete),
 		              NULL, NULL,
 		              g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
-	
+
 	signals[POPUP] = 
 		g_signal_new ("popup",
 		              G_OBJECT_CLASS_TYPE (gobject_class),
@@ -247,7 +232,7 @@ xpad_toolbar_class_init (XpadToolbarClass *klass)
 		              G_STRUCT_OFFSET (XpadToolbarClass, popup),
 		              NULL, NULL,
 		              g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, GTK_TYPE_MENU);
-	
+
 	signals[POPDOWN] = 
 		g_signal_new ("popdown",
 		              G_OBJECT_CLASS_TYPE (gobject_class),
@@ -259,7 +244,7 @@ xpad_toolbar_class_init (XpadToolbarClass *klass)
 	g_object_class_install_property (gobject_class,
 					 PROP_PAD,
 					 g_param_spec_pointer ("pad",
-									"Pad ",
+									"Pad",
 									"Pad associated with this toolbar",
 									G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 }
@@ -272,37 +257,35 @@ xpad_toolbar_init (XpadToolbar *toolbar)
 	toolbar->priv->move_motion_handler = 0;
 	toolbar->priv->move_button_release_handler = 0;
 	toolbar->priv->move_key_press_handler = 0;
-	
-	g_object_set (G_OBJECT (toolbar),
+
+	g_object_set (toolbar,
 	              "icon-size", GTK_ICON_SIZE_MENU,
 	              "show-arrow", FALSE,
 	              "toolbar-style", GTK_TOOLBAR_ICONS,
 	              NULL);
-	
-	g_signal_connect_swapped (xpad_settings (), "change-buttons", G_CALLBACK (xpad_toolbar_change_buttons), toolbar);
-	
+}
+
+static void xpad_toolbar_constructed (GObject *object)
+{
+	XpadSettings *settings;
+	XpadToolbar *toolbar = XPAD_TOOLBAR (object);
+
 	xpad_toolbar_change_buttons (toolbar);
+
+	g_object_get (toolbar->priv->pad, "settings", &settings, NULL);
+	g_signal_connect_swapped (settings, "change-buttons", G_CALLBACK (xpad_toolbar_change_buttons), toolbar);
 }
 
 static void
 xpad_toolbar_dispose (GObject *object)
 {
 	XpadToolbar *toolbar = XPAD_TOOLBAR (object);
-	
-	if (toolbar->priv->pad) {
-		g_object_unref (toolbar->priv->pad);
-		toolbar->priv->pad = NULL;
-	}
+
+	g_clear_object (&toolbar->priv->pad);
 
 	G_OBJECT_CLASS (xpad_toolbar_parent_class)->dispose (object);
 }
 
-static void
-xpad_toolbar_finalize (GObject *object)
-{
-	G_OBJECT_CLASS (xpad_toolbar_parent_class)->finalize (object);
-}
-
 void
 xpad_toolbar_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
@@ -311,13 +294,13 @@ xpad_toolbar_set_property (GObject *object, guint prop_id, const GValue *value,
 	switch (prop_id)
 	{
 	case PROP_PAD:
-		if (toolbar->priv->pad && G_IS_OBJECT (toolbar->priv->pad))
-			g_object_unref (toolbar->priv->pad);
-		if (G_VALUE_HOLDS_POINTER (value) && G_IS_OBJECT (g_value_get_pointer (value)))
-		{
+		g_clear_object (&toolbar->priv->pad);
+
+		if (G_VALUE_HOLDS_POINTER (value) && G_IS_OBJECT (g_value_get_pointer (value))) {
 			toolbar->priv->pad = g_value_get_pointer (value);
 			g_object_ref (toolbar->priv->pad);
 		}
+
 		break;
 
 	default:
@@ -343,35 +326,38 @@ xpad_toolbar_get_property (GObject *object, guint prop_id, GValue *value, GParam
 	}
 }
 
-static G_CONST_RETURN XpadToolbarButton *
+static const XpadToolbarButton *
 xpad_toolbar_button_lookup (XpadToolbar *toolbar, const gchar *name)
 {
+	/* A dirty way to silence the compiler for these unused variables. */
+	(void) toolbar;
+
 	guint i;
 	for (i = 0; i < G_N_ELEMENTS (buttons); i++)
 		if (!g_ascii_strcasecmp (name, buttons[i].name))
 			return &buttons[i];
-	
+
 	return NULL;
 }
 
 static GtkToolItem *
 xpad_toolbar_button_to_item (XpadToolbar *toolbar, const XpadToolbarButton *button)
 {
-	GtkToolItem *item;
-	// GtkWidget *child;
+	GtkToolItem *item = GTK_TOOL_ITEM (g_object_get_data (G_OBJECT (toolbar), button->name));
 
-	item = GTK_TOOL_ITEM (g_object_get_data (G_OBJECT (toolbar), button->name));
 	if (item)
 		return item;
 
 	switch (button->type)
 	{
 	case XPAD_BUTTON_TYPE_BUTTON:
-		item = GTK_TOOL_ITEM (gtk_tool_button_new_from_stock (button->stock));
+		item = gtk_tool_button_new (NULL, button->name);
+		gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON (item), button->icon_name);
 		g_signal_connect (item, "clicked", G_CALLBACK (xpad_toolbar_button_activated), NULL);
 		break;
 	case XPAD_BUTTON_TYPE_TOGGLE:
-		item = GTK_TOOL_ITEM (gtk_toggle_tool_button_new_from_stock (button->stock));
+		item = gtk_tool_button_new (NULL, button->name);
+		gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON (item), button->icon_name);
 		g_signal_connect (item, "toggled", G_CALLBACK (xpad_toolbar_button_activated), NULL);
 		break;
 	case XPAD_BUTTON_TYPE_SEPARATOR:
@@ -380,12 +366,12 @@ xpad_toolbar_button_to_item (XpadToolbar *toolbar, const XpadToolbarButton *butt
 	default:
 		return NULL;
 	}
-	
+
 	g_object_set_data (G_OBJECT (item), "xpad-toolbar", toolbar);
 	g_object_set_data (G_OBJECT (item), "xpad-tb", (gpointer) button);
 
 	g_object_set_data (G_OBJECT (toolbar), button->name, item);
-	
+
 	if (button->desc)
 		gtk_tool_item_set_tooltip_text (item, _(button->desc));
 
@@ -397,10 +383,10 @@ xpad_toolbar_button_activated (GtkToolButton *button)
 {
 	XpadToolbar *toolbar;
 	const XpadToolbarButton *tb;
-	
+
 	toolbar = XPAD_TOOLBAR (g_object_get_data (G_OBJECT (button), "xpad-toolbar"));
 	tb = (const XpadToolbarButton *) g_object_get_data (G_OBJECT (button), "xpad-tb");
-	
+
 	g_signal_emit (toolbar, signals[tb->signal], 0);
 }
 
@@ -409,32 +395,38 @@ xpad_toolbar_change_buttons (XpadToolbar *toolbar)
 {
 	GList *list, *temp;
 	const GSList *slist, *stemp;
-	gint i = 0;
-	guint j = 0;
+	guint i = 0, j = 0;
 	GtkToolItem *item;
-	
+	XpadSettings *settings;
+
 	list = gtk_container_get_children (GTK_CONTAINER (toolbar));
-	
+
 	for (temp = list; temp; temp = temp->next)
 		gtk_widget_destroy (temp->data);
-	
+
 	g_list_free (list);
 	g_list_free (temp);
 
 	for (j = 0; j < G_N_ELEMENTS (buttons); j++)
 		g_object_set_data (G_OBJECT (toolbar), buttons[j].name, NULL);
-	
-	slist = xpad_settings_get_toolbar_buttons (xpad_settings ());
+
+	g_object_get (toolbar->priv->pad, "settings", &settings, NULL);
+
+	slist = xpad_settings_get_toolbar_buttons (settings);
+
 	for (stemp = slist; stemp; stemp = stemp->next)
 	{
 		const XpadToolbarButton *button;
-		
+
 		button = xpad_toolbar_button_lookup (toolbar, stemp->data);
-		if (!button)
+
+		if (button->type == XPAD_BUTTON_TYPE_SEPARATOR)
+			item = gtk_separator_tool_item_new ();
+		else if (button->type == XPAD_BUTTON_TYPE_BUTTON)
+			item = xpad_toolbar_button_to_item (toolbar, button);
+		else
 			continue;
-		
-		item = xpad_toolbar_button_to_item (toolbar, button);
-		
+
 		if (item)
 		{
 			g_object_set_data (G_OBJECT (item), "xpad-button-num", GINT_TO_POINTER (i));
@@ -443,7 +435,7 @@ xpad_toolbar_change_buttons (XpadToolbar *toolbar)
 			i++;
 		}
 	}
-	
+
 	item = gtk_separator_tool_item_new ();
 	g_object_set_data (G_OBJECT (item), "xpad-button-num", GINT_TO_POINTER (i));
 	gtk_separator_tool_item_set_draw (GTK_SEPARATOR_TOOL_ITEM (item), FALSE);
@@ -451,7 +443,7 @@ xpad_toolbar_change_buttons (XpadToolbar *toolbar)
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
 	gtk_widget_show_all (GTK_WIDGET (item));
 	i++;
-	
+
 	item = xpad_grip_tool_item_new ();
 	g_object_set_data (G_OBJECT (item), "xpad-button-num", GINT_TO_POINTER (i));
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
@@ -466,21 +458,9 @@ xpad_toolbar_change_buttons (XpadToolbar *toolbar)
 }
 
 static void
-xpad_toolbar_remove_all_buttons ()
+xpad_toolbar_add_button (GtkMenuItem *menu_item, XpadSettings *settings)
 {
-	xpad_settings_remove_all_toolbar_buttons (xpad_settings ());
-}
-
-static void
-xpad_toolbar_remove_last_button ()
-{
-	xpad_settings_remove_last_toolbar_button (xpad_settings ());
-}
-
-static void
-xpad_toolbar_add_button (const gchar *name)
-{
-	xpad_settings_add_toolbar_button (xpad_settings (), name);
+	xpad_settings_add_toolbar_button (settings, gtk_menu_item_get_accel_path (menu_item) + 2);
 }
 
 static void
@@ -492,83 +472,64 @@ menu_deactivated (GtkWidget *menu, GtkToolbar *toolbar)
 static gboolean
 xpad_toolbar_popup_context_menu (GtkToolbar *toolbar, gint x, gint y, gint button)
 {
-	GtkWidget *menu;
-	const GSList *current_buttons;
-	guint i;
-	
-	menu = gtk_menu_new ();
-	
-	current_buttons = xpad_settings_get_toolbar_buttons (xpad_settings ());
+	/* A dirty way to silence the compiler for these unused variables. */
+	(void) x;
+	(void) y;
 
+	guint i;
 	gboolean is_button = FALSE;
-	
+	XpadSettings *settings;
+
+	g_object_get (XPAD_TOOLBAR (toolbar)->priv->pad, "settings", &settings, NULL);
+	const GSList *current_buttons = xpad_settings_get_toolbar_buttons (settings);
+	GtkMenu *menu = GTK_MENU (gtk_menu_new ());
+
 	for (i = 0; i < G_N_ELEMENTS (buttons); i++)
 	{
 		const GSList *j;
-		GtkWidget *item, *image;
-		
-		if (strcmp (buttons[i].name, "sep") != 0)
+		GtkWidget *item;
+
+		for (j = current_buttons; j; j = j->next)
+			if (g_ascii_strcasecmp (j->data, "Separator") && !g_ascii_strcasecmp (j->data, buttons[i].name))
+				break;
+
+		if (j)
 		{
-			for (j = current_buttons; j; j = j->next)
-				if (!g_ascii_strcasecmp (j->data, buttons[i].name))
-					break;
-			
-			if (j)
-			{
-				is_button = TRUE;
-				continue;
-			}
-		}
-		else
-		{
-			/* Don't let user add separators until we can allow clicks on them. */
+			is_button = TRUE;
 			continue;
 		}
-		
-		item = gtk_image_menu_item_new_with_mnemonic (buttons[i].menu_desc);
-		
-		image = gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_MENU);
-		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-		
-		g_signal_connect_swapped (item, "activate", G_CALLBACK (xpad_toolbar_add_button), (gpointer) buttons[i].name);
-		
-		gtk_menu_attach (GTK_MENU (menu), item, 0, 1, i, i + 1);
+
+		item = gtk_menu_item_new_with_mnemonic (buttons[i].menu_desc);
+		/* Ugly workaround by abusing the accel_path to get the variable passed to the next function */
+		gtk_menu_item_set_accel_path (GTK_MENU_ITEM (item), g_strdup_printf ("</%s", buttons[i].name));
+		g_signal_connect (item, "activate", G_CALLBACK (xpad_toolbar_add_button), settings);
+		gtk_menu_attach (menu, item, 0, 1, i, i + 1);
 		gtk_widget_show (item);
 	}
 
 	if (is_button)
 	{
-		GtkWidget *item, *image;
+		GtkWidget *item;
 
-		item = gtk_image_menu_item_new_with_mnemonic (N_("Remove All _Buttons"));
-		
-		image = gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_MENU);
-		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-		
-		g_signal_connect_swapped (item, "activate", G_CALLBACK (xpad_toolbar_remove_all_buttons), NULL);
-		
-		gtk_menu_attach (GTK_MENU (menu), item, 0, 1, i, i + 1);
+		item = gtk_menu_item_new_with_mnemonic (N_("Remove All _Buttons"));
+		g_signal_connect_swapped (item, "activate", G_CALLBACK (xpad_settings_remove_all_toolbar_buttons), settings);
+		gtk_menu_attach (menu, item, 0, 1, i, i + 1);
 		gtk_widget_show (item);
 
 		i++;
-		
-		item = gtk_image_menu_item_new_with_mnemonic (N_("Remo_ve Last Button"));
-		
-		image = gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_MENU);
-		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-		
-		g_signal_connect_swapped (item, "activate", G_CALLBACK (xpad_toolbar_remove_last_button), NULL);
-		
-		gtk_menu_attach (GTK_MENU (menu), item, 0, 1, i, i + 1);
+
+		item = gtk_menu_item_new_with_mnemonic (N_("Remo_ve Last Button"));
+		g_signal_connect_swapped (item, "activate", G_CALLBACK (xpad_settings_remove_last_toolbar_button), settings);
+		gtk_menu_attach (menu, item, 0, 1, i, i + 1);
 		gtk_widget_show (item);
 	}
-	
+
 	g_signal_connect (menu, "deactivate", G_CALLBACK (menu_deactivated), toolbar);
-	
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, (guint) ((button < 0) ? 0 : button), gtk_get_current_event_time ());
-	
+
+	gtk_menu_popup_at_pointer (menu, NULL);
+
 	g_signal_emit (toolbar, signals[POPUP], 0, menu);
-	
+
 	return TRUE;
 }
 
@@ -615,216 +576,3 @@ xpad_toolbar_enable_paste_button (XpadToolbar *toolbar, gboolean enable)
 	const XpadToolbarButton *button = xpad_toolbar_button_lookup (toolbar, "Paste");
 	xpad_toolbar_enable_button (toolbar, button, enable);
 }
-
-/*
- * The code below was code in progress of a previous developer
-static gboolean
-xpad_toolbar_button_press_event (GtkWidget *widget, GdkEventButton *event)
-{
-	// Ignore double-clicks and triple-clicks
-	if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
-	{
-		XpadToolbar *toolbar = XPAD_TOOLBAR (g_object_get_data (G_OBJECT (widget), "xpad-toolbar"));
-		xpad_toolbar_popup_button_menu (widget, event, toolbar);
-		return TRUE;
-	}
-	else if (event->button == 2 && event->type == GDK_BUTTON_PRESS)
-	{
-		XpadToolbar *toolbar = XPAD_TOOLBAR (g_object_get_data (G_OBJECT (widget), "xpad-toolbar"));
-		xpad_toolbar_move_button_start (toolbar, widget);
-		return TRUE;
-	}
-
-	return FALSE;
-}
-
-static gboolean
-xpad_toolbar_popup_button_menu (GtkWidget *button, GdkEventButton *event, XpadToolbar *toolbar)
-{
-	GtkWidget *menu;
-	GtkWidget *item, *image;
-
-	menu = gtk_menu_new ();
-
-
-	item = gtk_image_menu_item_new_with_mnemonic (_("_Remove From Toolbar"));
-
-	image = gtk_image_new_from_stock (GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-
-	g_signal_connect_swapped (item, "activate", G_CALLBACK (xpad_toolbar_remove_button), button);
-	gtk_menu_attach (GTK_MENU (menu), item, 0, 1, 0, 1);
-	gtk_widget_show (item);
-
-
-	item = gtk_menu_item_new_with_mnemonic (_("_Move"));
-	g_signal_connect_swapped (item, "activate", G_CALLBACK (move_menu_item_activated), button);
-	gtk_menu_attach (GTK_MENU (menu), item, 0, 1, 1, 2);
-	gtk_widget_show (item);
-
-
-	g_signal_connect (menu, "deactivate", G_CALLBACK (menu_deactivated), toolbar);
-
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, event ? event->button : 0, gtk_get_current_event_time ());
-
-	g_signal_emit (toolbar, signals[POPUP], 0, menu);
-
-	return TRUE;
-}
-
-static void
-xpad_toolbar_remove_button (GtkWidget *button)
-{
-	gint button_num;
-
-	button_num = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (button), "xpad-button-num"));
-
-	xpad_settings_remove_toolbar_button (xpad_settings (), button_num);
-}
-
-static void
-move_menu_item_activated (GtkWidget *button)
-{
-	XpadToolbar *toolbar;
-
-	toolbar = XPAD_TOOLBAR (g_object_get_data (G_OBJECT (button), "xpad-toolbar"));
-
-	xpad_toolbar_move_button_start (toolbar, button);
-}
-
-static gboolean
-xpad_toolbar_move_button_start (XpadToolbar *toolbar, GtkWidget *button)
-{
-	GdkGrabStatus  status;
-	GdkCursor     *fleur_cursor;
-	GtkWidget *widget;
-
-	widget = GTK_WIDGET (toolbar);
-	gtk_grab_add (widget);
-
-	fleur_cursor = gdk_cursor_new (GDK_FLEUR);
-
-	// g_object_ref (button);
-	toolbar->priv->move_removed = FALSE;
-	toolbar->priv->move_button = GTK_TOOL_ITEM (button);
-	toolbar->priv->move_index = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (button), "xpad-button-num"));
-
-	toolbar->priv->move_button_release_handler = g_signal_connect (toolbar, "button-release-event", G_CALLBACK (xpad_toolbar_move_button_end), NULL);
-	toolbar->priv->move_key_press_handler = g_signal_connect (toolbar, "key-press-event", G_CALLBACK (xpad_toolbar_move_button_move_keyboard), NULL);
-	toolbar->priv->move_motion_handler = g_signal_connect (toolbar, "motion-notify-event", G_CALLBACK (xpad_toolbar_move_button_move), NULL);
-
-	status = gdk_pointer_grab (widget->window, FALSE,
-				   GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK, NULL,
-				   fleur_cursor, gtk_get_current_event_time ());
-
-	gdk_cursor_unref (fleur_cursor);
-	gdk_flush ();
-
-	if (status != GDK_GRAB_SUCCESS)
-	{
-		xpad_toolbar_move_button_end (toolbar);
-	}
-
-	return TRUE;
-}
-
-static gboolean
-xpad_toolbar_move_button_move_keyboard (XpadToolbar *toolbar, GdkEventKey *event)
-{
-	if (event->keyval == GDK_Left || event->keyval == GDK_KP_Left)
-	{
-		if (!toolbar->priv->move_removed)
-		{
-			toolbar->priv->move_removed = TRUE;
-			gtk_container_remove (GTK_CONTAINER (toolbar), GTK_WIDGET (toolbar->priv->move_button));
-		}
-
-		if (toolbar->priv->move_index > 0)
-			toolbar->priv->move_index--;
-
-		gtk_toolbar_set_drop_highlight_item (GTK_TOOLBAR (toolbar), toolbar->priv->move_button, toolbar->priv->move_index);
-	}
-	else if (event->keyval == GDK_Right || event->keyval == GDK_KP_Right)
-	{
-		gint max;
-
-		if (!toolbar->priv->move_removed)
-		{
-			toolbar->priv->move_removed = TRUE;
-			gtk_container_remove (GTK_CONTAINER (toolbar), GTK_WIDGET (toolbar->priv->move_button));
-		}
-
-		max = gtk_toolbar_get_n_items (GTK_TOOLBAR (toolbar)) - 2;
-
-		if (toolbar->priv->move_index < max)
-			toolbar->priv->move_index++;
-
-		gtk_toolbar_set_drop_highlight_item (GTK_TOOLBAR (toolbar), toolbar->priv->move_button, toolbar->priv->move_index);
-	}
-	else if (event->keyval == GDK_space || event->keyval == GDK_KP_Space || event->keyval == GDK_Return || event->keyval == GDK_KP_Enter)
-	{
-		xpad_toolbar_move_button_end (toolbar);
-		return TRUE;
-	}
-
-	return TRUE;
-}
-
-static gboolean
-xpad_toolbar_move_button_move (XpadToolbar *toolbar, GdkEventMotion *event)
-{
-	gint max;
-
-	if (!toolbar->priv->move_removed)
-	{
-		toolbar->priv->move_removed = TRUE;
-		gtk_container_remove (GTK_CONTAINER (toolbar), GTK_WIDGET (toolbar->priv->move_button));
-	}
-
-	toolbar->priv->move_index = gtk_toolbar_get_drop_index (GTK_TOOLBAR (toolbar), event->x, event->y);
-
-	// Must not move past separator or grip
-	max = gtk_toolbar_get_n_items (GTK_TOOLBAR (toolbar)) - 2;
-	toolbar->priv->move_index = MIN (toolbar->priv->move_index, max);
-
-	gtk_toolbar_set_drop_highlight_item (GTK_TOOLBAR (toolbar), toolbar->priv->move_button, toolbar->priv->move_index);
-
-	return TRUE;
-}
-
-static gboolean
-xpad_toolbar_move_button_end (XpadToolbar *toolbar)
-{
-	gint old_spot;
-	gint max;
-
-	g_signal_handler_disconnect (toolbar, toolbar->priv->move_button_release_handler);
-	g_signal_handler_disconnect (toolbar, toolbar->priv->move_key_press_handler);
-	g_signal_handler_disconnect (toolbar, toolbar->priv->move_motion_handler);
-	toolbar->priv->move_button_release_handler = 0;
-	toolbar->priv->move_key_press_handler = 0;
-	toolbar->priv->move_motion_handler = 0;
-
-	gtk_toolbar_set_drop_highlight_item (GTK_TOOLBAR (toolbar), NULL, 0);
-
-	old_spot = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (toolbar->priv->move_button), "xpad-button-num"));
-
-	// Must not move past separator or grip
-	max = gtk_toolbar_get_n_items (GTK_TOOLBAR (toolbar)) - 2;
-	toolbar->priv->move_index = MIN (toolbar->priv->move_index, max);
-
-	if (!xpad_settings_move_toolbar_button (xpad_settings (), old_spot,	toolbar->priv->move_index) &&
-	    toolbar->priv->move_removed)
-	{
-		gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolbar->priv->move_button, toolbar->priv->move_index);
-	}
-
-	// g_object_unref (toolbar->priv->move_button);
-	toolbar->priv->move_button = NULL;
-
-	gtk_grab_remove (GTK_WIDGET (toolbar));
-	gdk_pointer_ungrab (gtk_get_current_event_time ());
-	return TRUE;
-}
-
-*/
